import type { PeerId } from "@libp2p/interface-peer-id";
import { Libp2p } from "libp2p";
import { DataMessage, DirectStream, DirectStreamOptions, Message, PeerStreams } from "@dao-xyz/libp2p-direct-stream";
import { PubSubMessage, PubSubData, toUint8Array, GetSubscribers, Subscription } from "./messages.js";
import { PublicSignKey } from "@dao-xyz/peerbit-crypto";
import type { Connection } from "@libp2p/interface-connection";
import { Goodbye } from "@dao-xyz/libp2p-direct-stream";
import { PeerEvents } from "@dao-xyz/libp2p-direct-stream";
export { PubSubMessage, PubSubData, toUint8Array, GetSubscribers, Subscription, };
export declare const logger: import("pino").Logger<import("pino").LoggerOptions | import("pino").DestinationStream>;
export interface PeerStreamsInit {
    id: PeerId;
    protocol: string;
}
interface From {
    from: PublicSignKey;
}
export interface SubscriptionEvent extends From {
    subscriptions: Subscription[];
}
export interface UnsubcriptionEvent extends From {
    unsubscriptions: Subscription[];
}
export interface PubSubEvents extends PeerEvents {
    data: CustomEvent<PubSubData>;
    subscribe: CustomEvent<SubscriptionEvent>;
    unsubscribe: CustomEvent<UnsubcriptionEvent>;
    message: CustomEvent<Message>;
}
export type DirectSubOptions = {
    aggregate: boolean;
};
export type SubscriptionData = {
    timestamp: bigint;
    data?: Uint8Array;
};
export declare class DirectSub extends DirectStream<PubSubEvents> {
    topics: Map<string, Map<string, SubscriptionData>>;
    peerToTopic: Map<string, Set<string>>;
    topicsToStreams: Map<string, Set<string>>;
    subscriptions: Map<string, {
        counter: number;
        data?: Uint8Array;
    }>;
    constructor(libp2p: Libp2p, props?: DirectStreamOptions);
    stop(): Promise<void>;
    onPeerConnected(peerId: PeerId, conn: Connection): Promise<any[] | undefined>;
    onPeerDisconnected(peerId: PeerId): Promise<void>;
    private initializeTopic;
    private initializePeer;
    /**
     * Subscribes to a given topic.
     */
    /**
     * @param topic,
     * @param data, metadata associated with the subscription, shared with peers
     */
    subscribe(topic: string | string[], options?: {
        data?: Uint8Array;
    }): Promise<void>;
    /**
     *
     * @param topic
     * @param force
     * @returns true unsubscribed completely
     */
    unsubscribe(topic: string, options?: {
        force: boolean;
        data: Uint8Array;
    }): Promise<boolean>;
    getSubscribers(topic: string): Map<string, SubscriptionData> | undefined;
    getSubscribersWithData(topic: string, data: Uint8Array, options?: {
        prefix: boolean;
    }): string[] | undefined;
    listenForSubscribers(topic: string): void;
    requestSubscribers(topic: string | string[], streams?: PeerStreams[]): Promise<void>;
    getNeighboursWithTopics(topics: string[], otherPeers?: string[]): Set<string>;
    getStreamsWithTopics(topics: string[], otherPeers?: string[]): PeerStreams[];
    publish(data: Uint8Array, options: {
        topics?: string[];
        to?: (string | PeerId | PublicSignKey)[];
        strict?: false;
    } | {
        topics: string[];
        to: (string | PeerId | PublicSignKey)[];
        strict: true;
    }): Promise<void>;
    onGoodbye(from: PeerId, peerStream: PeerStreams, message: Goodbye): Promise<boolean>;
    deletePeerFromTopic(topic: string, publicKeyHash: string, fromStream?: PeerId): SubscriptionData | undefined;
    onPeerUnreachable(publicKey: PublicSignKey): void;
    onDataMessage(from: PeerId, stream: PeerStreams, message: DataMessage): Promise<boolean>;
}
